'use strict';

var _typeof = typeof Symbol === "function" && typeof Symbol.iterator === "symbol" ? function (obj) { return typeof obj; } : function (obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol ? "symbol" : typeof obj; };

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.editor = editor;

var _fsExtra = require('fs-extra');

var _fsExtra2 = _interopRequireDefault(_fsExtra);

var _extend = require('extend');

var _extend2 = _interopRequireDefault(_extend);

var _cli = require('../../cli');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var util = new _cli.Util();
var arrayBlock = [];
var text;
var json;
var fakeContent = false;

function add(type, key, desc, maxLength, tab, value, source, display) {
  var hasSource = false;

  if ((typeof type === 'undefined' ? 'undefined' : _typeof(type)) === 'object' && Object.prototype.toString.call(type) === '[object Object]') {
    var key = type.key.split('[')[0];
    var index = type.key.match(/[^\[]+?(?=\])/)[0];
    var prop = type.key.replace(/[^\.]+?\./, '');
    if (fakeContent) {
      value = util.lorem(type.type);
    }
    if (typeof json[key] !== 'undefined' && json[key] !== null && typeof json[key][index] !== 'undefined' && json[key][index] !== null && typeof json[key][index][prop] !== 'undefined' && json[key][index][prop] !== null) {
      value = json[key][index][prop];
    } else if (typeof value !== 'undefined' && value !== null && value !== '') {
      if (typeof json[key] === 'undefined' || json[key] === null) json[key] = [];
      if (typeof json[key][index] === 'undefined' || json[key][index] === null) json[key][index] = {};
      json[key][index][prop] = value;
    }

    util.add(type.type, type.key, type.desc, type.maxLength, type.tab, value, source);
  } else {
    if ((typeof value === 'undefined' || value === null || value === '') && fakeContent) {
      if (typeof source === 'undefined' || source === null) {
        value = util.lorem(type);
      } else {
        if ((typeof source === 'undefined' ? 'undefined' : _typeof(source)) === 'object') {
          value = [];
          var i = 0;
          Array.prototype.forEach.call(source, function (item) {
            if (typeof maxLength === 'undefined' || maxLength === null || maxLength === '' || i < maxLength) {
              value.push(item);
            }
            i++;
          });
          json[key] = value;
        } else {
          value = source;
        }
      }
      json[key] = value;
    }

    util.add(type, key, desc, maxLength, tab, value, source, display);
  }

  return value;
}

function addToForm(match) {
  var keyArray = arguments.length <= 1 || arguments[1] === undefined ? null : arguments[1];
  var i = arguments.length <= 2 || arguments[2] === undefined ? 0 : arguments[2];

  var v = match // the match
  ,
      key = (0, _cli.getAttr)(v, 'key'),
      type = (0, _cli.getAttr)(v, 'type'),
      desc = (0, _cli.getAttr)(v, 'desc'),
      maxLength = (0, _cli.getAttr)(v, 'max-length'),
      tab = (0, _cli.getAttr)(v, 'tab');

  if (typeof keyArray !== 'undefined' && keyArray !== null) {
    var realKey = key.replace(/[^\.]+?\./, '');

    if (key.indexOf(keyArray + '.') >= 0 && realKey.length > 0) {
      insertAbeEach(keyArray, realKey, type, desc, maxLength, i, tab);
    } else if (util.dontHaveKey(key)) {
      json[key] = add(type, key, desc, maxLength, tab, json[key], tab);
    }
  } else if (util.dontHaveKey(key) && util.isSingleAbe(v, text)) {
    var realKey = key.replace(/\./g, '-');
    json[realKey] = add(type, key, desc, maxLength, tab, json[key]);
  }
}

function matchAttrAbe() {
  var patt = /abe [^{{}}]+?(?=\}})/g,
      match;
  // While regexp match HandlebarsJS template item => keepgoing
  while (match = patt.exec(text)) {
    addToForm(match[0]);
  }
}

function insertAbeEach(keyArray, realKey, type, desc, maxLength, i, tab, source) {
  if (typeof arrayBlock[keyArray][realKey] === "undefined" || arrayBlock[keyArray][realKey] === null) {
    arrayBlock[keyArray][realKey] = [];
  }
  var key = keyArray + "[" + i + "]." + realKey;
  var exist = false;
  Array.prototype.forEach.call(arrayBlock[keyArray][realKey], function (block) {
    if (block.key === key) {
      exist = true;
    }
  });
  if (!exist) {
    arrayBlock[keyArray][realKey].push({
      type: type,
      key: keyArray + "[" + i + "]." + realKey,
      desc: desc + " " + i,
      maxLength: maxLength,
      placeholder: '',
      tab: tab
    });
  }
}

function each() {
  var pattEach = /(\{\{#each (\r|\t|\n|.)*?\/each\}\})/g;
  var patt = /abe [^{{}}]+?(?=\}})/g;
  var textEach, match;

  while (textEach = pattEach.exec(text)) {
    var keyArray = textEach[0].match(/#each (\n|.)*?\}/);
    keyArray = keyArray[0].slice(6, keyArray[0].length - 1);

    if (keyArray.split(' ').length > 1) keyArray = keyArray.split(' ')[0];
    arrayBlock[keyArray] = [];
    // ce while boucle sur les block de contenu {{abe}}
    while (match = patt.exec(textEach[0])) {
      var v = match[0];

      if (v.indexOf('abe') > -1) {
        if (json[keyArray]) {
          for (var i = 0; i < json[keyArray].length; i++) {
            var key = json[keyArray];
            addToForm(v, keyArray, i);
          }
        } else {
          addToForm(v, keyArray, 0);
        }
      }
    }

    // ici on boucle a nouveau sur les champs pour les placer a la suite dans le formulaire
    var attrArray = [],
        length = 0;
    for (var index in arrayBlock[keyArray]) {
      attrArray.push(index);
      length = arrayBlock[keyArray][index].length;
    }

    for (var i = 0; i < length; i++) {
      for (var j = 0; j < attrArray.length; j++) {
        add(arrayBlock[keyArray][attrArray[j]][i]);
      }
    }
  }
}

function addSource() {
  var listReg = /({{abe.*type=[\'|\"]data.*}})/g,
      match,
      limit = 0;

  while (match = listReg.exec(text)) {
    var type = 'text',
        editable = (0, _cli.getAttr)(match[0], 'editable'),
        key = (0, _cli.getAttr)(match[0], 'key'),
        display = (0, _cli.getAttr)(match[0], 'display'),
        desc = (0, _cli.getAttr)(match[0], 'desc'),
        maxLength = (0, _cli.getAttr)(match[0], 'max-length'),
        tab = (0, _cli.getAttr)(match[0], 'tab');

    editable = typeof editable === 'undefined' || editable === null || editable === '' || editable === 'false' ? false : true;

    var eachSource = new RegExp('{{#each ' + key + '}');
    maxLength = eachSource.test(text) ? maxLength : 1;
    var source = _cli.config.source.name;

    if (editable) {
      add(type, key, desc, maxLength, tab, json[key], json[source][key], display);
    } else {
      json[key] = json[source][key];
    }
  }
}

function orderBlock() {
  var formBlock = {};

  for (var tab in util.form) {
    var formBlockTab = {};
    for (var i = 0; i < util.form[tab].item.length; i++) {
      var blockName = util.form[tab].item[i].block === '' ? 'default' : util.form[tab].item[i].block;
      if (util.form[tab].item[i].key.indexOf('[') > -1) {
        blockName = util.form[tab].item[i].key.split('[')[0];
      }
      if (typeof formBlockTab[blockName] === 'undefined' || formBlockTab[blockName] === null) formBlockTab[blockName] = [];
      formBlockTab[blockName].push(util.form[tab].item[i]);
    }
    if (typeof formBlock[tab] === 'undefined' || formBlock[tab] === null) {
      formBlock[tab] = {};
    }
    formBlock[tab] = formBlockTab;
  }

  return formBlock;
}

function editor(fileName, tplUrl, fake) {
  fakeContent = fake;
  util = new _cli.Util();

  json = {};
  if (_cli.fileUtils.isFile(tplUrl.json.path)) {
    json = _fsExtra2.default.readJsonSync(tplUrl.json.path, 'utf8');
  }

  text = '';
  var extension = _cli.config.files.templates.extension;
  if (_cli.fileUtils.isFile(fileName + '.' + extension)) {
    text = _fsExtra2.default.readFileSync(fileName + '.' + extension, 'utf8');
  }
  _cli.Util.getDataList(_cli.FileParser.getWebsite(fileName), text, json);

  addSource();

  text = _cli.Util.removeDataList(text);

  matchAttrAbe(text, json);
  arrayBlock = [];
  each(text, json);
  _cli.log.delAndWrite('editor', 'json', json);

  var blocks = orderBlock();

  return {
    form: blocks,
    json: json
  };
}