'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _express = require('express');

var _express2 = _interopRequireDefault(_express);

var _fs = require('fs');

var _fs2 = _interopRequireDefault(_fs);

var _fsExtra = require('fs-extra');

var _fsExtra2 = _interopRequireDefault(_fsExtra);

var _mkdirp = require('mkdirp');

var _mkdirp2 = _interopRequireDefault(_mkdirp);

var _htmlMinifier = require('html-minifier');

var _extend = require('extend');

var _extend2 = _interopRequireDefault(_extend);

var _cli = require('../../cli');

var _editor = require('./editor');

var _abeLocale = require('../helpers/abe-locale');

var _abeLocale2 = _interopRequireDefault(_abeLocale);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var router = _express2.default.Router();

// Abe templating engine route
router.get('/abe/*', function (req, res) {
  var templatePath = _cli.fileUtils.getTemplatePath(req.params[0]);
  var filePath = _cli.fileUtils.getFilePath(req.query.filePath);
  var obj;
  var tplUrl;
  var isHome = true;
  var manager = {};

  if (templatePath !== null && filePath !== null) {

    isHome = false;

    if (!_cli.fileAttr.test(filePath)) {
      var folderFilePath = filePath.split('/');
      folderFilePath.pop();
      folderFilePath = _cli.fileUtils.pathWithRoot(folderFilePath.join('/'));
      _mkdirp2.default.sync(folderFilePath);
      var files = _cli.FileParser.getFiles(folderFilePath, true, 2);
      var latest = _cli.fileAttr.filterLatestVersion(_cli.fileAttr.getFilesRevision(files, filePath), 'draft');
      if (latest.length) {
        filePath = latest[0].path;
      }
    }

    tplUrl = _cli.FileParser.getFileDataFromUrl(filePath);
    var fakeContent = req.query.fakeContent ? true : false;

    obj = (0, _editor.editor)(templatePath, tplUrl, fakeContent);

    if (!_cli.fileUtils.isFile(tplUrl.json.path)) {
      var json = obj.json;
      var text = '';
      var tpl = templatePath;
      var extension = _cli.config.files.templates.extension;
      if (_cli.fileUtils.isFile(tpl + '.' + extension)) {
        text = _fsExtra2.default.readFileSync(tpl + '.' + extension, 'utf8');
      }
      text = _cli.Util.removeDataList(text);

      var resSave = (0, _cli.save)(filePath, req.params[0], json, text, 'draft');
      filePath = resSave.htmlPath;
    }

    _cli.FileParser.getAssetsFolder();
    _cli.FileParser.getAssets();

    var revisionFilePath = _cli.FileParser.changePathEnv(filePath, _cli.config.draft.url);
    var dirPath = _cli.fileUtils.removeLast(revisionFilePath);
    var allDraft = _cli.FileParser.getFiles(dirPath, true, 99, new RegExp("\\." + _cli.config.files.templates.extension));

    allDraft = _cli.FileParser.getMetas(allDraft, 'draft');
    var breadcrumb = req.params[0].split('/');
    manager.file = {
      revision: _cli.fileAttr.getFilesRevision(allDraft, revisionFilePath),
      template: breadcrumb,
      path: req.query.filePath ? _cli.fileUtils.cleanTplName(req.query.filePath) : ''
    };
    if (manager.file.revision.length > 0) {
      var publishPath = _cli.fileAttr.delete(manager.file.revision[0].path.replace(new RegExp('/' + _cli.config.draft.url + '/'), '/' + _cli.config.publish.url + '/'));
      manager.file.isPublished = _cli.fileUtils.isFile(publishPath);
    }
  }
  manager.home = {
    files: _cli.FileParser.getAllFiles()
  };

  manager.list = _cli.FileParser.getProjetFiles();
  manager.editConfig = _cli.config.getConfigByWebsite();
  manager.config = JSON.stringify(_cli.config);

  var _hasBlock = obj ? obj.hasBlock : false;
  var _hasSingleBlock = obj ? obj.hasSingleBlock : false;
  var _template = filePath ? '/page/' + req.params[0] + ('?filePath=' + req.query.filePath) : false;
  var _form = obj ? obj.form : false;
  var _json = obj ? obj.json : false;
  var _file = tplUrl ? tplUrl.draft.file : false;
  var _filePath = req.query.filePath ? req.query.filePath : false;

  // var textMinified = minify(text.replace(/'/g, "\\'"), {collapseWhitespace: true})
  res.render('../views/template-engine', { // render html and send ABE engine to users
    abeUrl: '/abe/',
    test: JSON.stringify(_abeLocale2.default),
    text: _abeLocale2.default,
    templatePath: req.params[0],
    template: _template,
    hasSingleBlock: _hasSingleBlock,
    hasBlock: _hasBlock,
    form: _form,
    urlToSaveFile: _filePath,
    tplName: _file,
    json: _json,
    config: _cli.config,
    manager: manager
  });
});

// Template page route
router.get('/page/*', function (req, res) {
  var templatePath = _cli.fileUtils.getTemplatePath(req.params[0]);
  var filePath = _cli.fileUtils.getFilePath(req.query.filePath);

  if (typeof filePath !== 'undefined' && filePath !== null) {

    var tplUrl = _cli.FileParser.getFileDataFromUrl(filePath);

    _cli.log.write('Index-route-page', 'templatePath', templatePath, 'tplUrl', tplUrl);

    var json = {};

    var resp = _fsExtra2.default.readJsonSync(tplUrl.json.path);

    if (typeof resp !== 'undefined' && resp !== null) {
      json = resp;
    }

    var meta = _cli.config.meta.name;
    var extension = _cli.config.files.templates.extension;

    var text = _fsExtra2.default.readFileSync(_cli.fileUtils.concatPath(_cli.config.root, _cli.config.templates.url, json[meta].template + '.' + extension), 'utf8');

    _cli.Util.getDataList(_cli.FileParser.getWebsite(filePath), text, json);
    _cli.log.write('data-list', text, json);
    text = _cli.Util.removeDataList(text);

    var page = new _cli.Page(templatePath, text, json);
    res.set('Content-Type', 'text/html');
    res.send(page.html);
  } else {
    // not 404 page if tag abe image upload into each block
    if (/upload%20image/g.test(req.url)) {
      var b64str = 'data:image/gif;base64,R0lGODlhAQABAAAAACH5BAEKAAEALAAAAAABAAEAAAICTAEAOw==';
      var img = new Buffer(b64str, 'base64');

      res.writeHead(200, {
        'Content-Type': 'image/jpeg',
        'Content-Length': img.length
      });
      res.end(img);
    } else {
      _cli.log.write('Index-route-page', '404 not found', 'filePath', filePath);
      res.status(404).send('Not found');
    }
  }
});

router.post('/save', function (req, res) {
  var saved = (0, _cli.save)(_cli.fileUtils.getFilePath(req.body.filePath), req.body.tplPath, req.body.json, '', req.body.saveAction);
  var result = {
    success: 1,
    json: saved.json
  };
  res.set('Content-Type', 'application/json');
  res.send(JSON.stringify(result));
});

router.get('/save-config', function (req, res) {
  _cli.config.set(req.query.website, req.query.json);
  res.set('Content-Type', 'application/json');
  res.send(JSON.stringify(req.query.json));
});

router.get('/unpublish', function (req, res) {
  var filePath = req.query.filePath;
  var dirPath = _cli.FileParser.unpublishFile(filePath);

  var result = {
    success: 1,
    file: req.query.filePath
  };
  res.set('Content-Type', 'application/json');
  res.send(JSON.stringify(result));
});

router.get('/delete', function (req, res) {
  var filePath = req.query.filePath;
  var dirPath = _cli.FileParser.deleteFile(filePath);

  var result = {
    success: 1,
    file: req.query.filePath
  };
  res.set('Content-Type', 'application/json');
  res.send(JSON.stringify(result));
});

router.get('/', function (req, res) {
  res.redirect('/abe/');
});

router.post('/upload/*', function (req, res) {
  var resp = { success: 1 };
  var filePath;
  var fstream;
  var folderWebPath = '/' + _cli.config.upload.image;
  var folderFilePath = _cli.fileUtils.concatPath(_cli.config.root, _cli.config.publish.url, folderWebPath);
  _mkdirp2.default.sync(folderFilePath);

  req.pipe(req.busboy);
  req.busboy.on('file', function (fieldname, file, filename) {
    filePath = _cli.fileUtils.concatPath(folderFilePath, filename);
    resp['filePath'] = _cli.fileUtils.concatPath(folderWebPath, filename);
    fstream = _fs2.default.createWriteStream(filePath);
    file.pipe(fstream);
    fstream.on('close', function () {});
  });
  req.busboy.on('finish', function () {
    res.set('Content-Type', 'application/json');
    res.send(JSON.stringify(resp));
  });
});

var site = new _cli.serveSite();

router.get('/site*', function (req, res) {
  if (!site.isStarted) site.start(_cli.config);
  res.set('Content-Type', 'application/json');
  res.send(JSON.stringify(site.infos));
});

exports.default = router;